@@ -7,7 +7,6 @@ from vtk.util.numpy_support import numpy_to_vtk, vtk_to_numpy
 import vtkplotter.utils as utils
 import vtkplotter.colors as colors
 import vtkplotter.shapes as shapes
-
 from vtkplotter.assembly import Assembly
 from vtkplotter.mesh import Mesh
 from vtkplotter.volume import Volume
@@ -23,9 +22,7 @@ Defines methods useful to analyse 3D meshes.
 __all__ = [
     "delaunay2D",
     "delaunay3D",
-    "extrude",
     "normalLines",
-    "extractLargestRegion",
     "alignLandmarks",
     "alignICP",
     "alignProcrustes",
@@ -42,40 +39,28 @@ __all__ = [
     "probeLine",
     "probePlane",
     "resampleArrays",
-    "volumeOperation",
     "recoSurface",
     "cluster",
     "removeOutliers",
-    "thinPlateSpline",
-    "meshQuality",
     "pointSampler",
     "geodesic",
     "convexHull",
     "mesh2Volume",
-    "splitByConnectivity",
     "projectSphereFilter",
-    "extractSurface",
     "voronoi3D",
     "connectedPoints",
     "interpolateToVolume",
     "interpolateToStructuredGrid",
     "streamLines",
     "densifyCloud",
-    "frequencyPassFilter",
     "implicitModeller",
     "signedDistanceFromPointCloud",
     "volumeFromMesh",
-    "computeNormalsWithPCA",
     "pointDensity",
-    "erodeVolume",
-    "dilateVolume",
-    "euclideanDistanceVolume",
-    "volumeToPoints",
-    "volumeCorrelation",
-    "warpMeshToPoint",
     "rectilinearGridToTetrahedra",
     "extractCellsByType",
     "pointCloudFrom",
+    "pointDensity",
 ]
 
 
@@ -112,46 +97,7 @@ def delaunay3D(dataset, alpha=0, tol=None, boundary=True):
     deln.Update()
     return deln.GetOutput()
 
-
-def extrude(mesh, zshift=1, rotation=0, dR=0, cap=True, res=1):
-    """
-    Sweep a polygonal data creating a "skirt" from free edges and lines, and lines from vertices.
-    The input dataset is swept around the z-axis to create new polygonal primitives.
-    For example, sweeping a line results in a cylindrical shell, and sweeping a circle creates a torus.
-
-    You can control whether the sweep of a 2D object (i.e., polygon or triangle strip)
-    is capped with the generating geometry.
-    Also, you can control the angle of rotation, and whether translation along the z-axis
-    is performed along with the rotation. (Translation is useful for creating "springs").
-    You also can adjust the radius of the generating geometry using the "dR" keyword.
-
-    The skirt is generated by locating certain topological features.
-    Free edges (edges of polygons or triangle strips only used by one polygon or triangle strips)
-    generate surfaces. This is true also of lines or polylines. Vertices generate lines.
-
-    This filter can be used to model axisymmetric objects like cylinders, bottles, and wine glasses;
-    or translational/rotational symmetric objects like springs or corkscrews.
-
-    Warning:
-
-        Some polygonal objects have no free edges (e.g., sphere). When swept, this will result
-        in two separate surfaces if capping is on, or no surface if capping is off.
-
-    |extrude| |extrude.py|_
-    """
-    rf = vtk.vtkRotationalExtrusionFilter()
-    rf.SetInputData(mesh.polydata())
-    rf.SetResolution(res)
-    rf.SetCapping(cap)
-    rf.SetAngle(rotation)
-    rf.SetTranslation(zshift)
-    rf.SetDeltaRadius(dR)
-    rf.Update()
-    act = Mesh(rf.GetOutput(), c=mesh.c(), alpha=mesh.alpha())
-    return act.computeNormals(cells=False).phong()
-
-
-def normalLines(mesh, ratio=1, atCells=True):
+def normalLines(mesh, ratio=1, atCells=True, scale=1):
     """
     Build an ``Mesh`` made of the normals at cells shown as lines.
 
@@ -181,7 +127,7 @@ def normalLines(mesh, ratio=1, atCells=True):
     glyph.SetVectorModeToUseNormal()
 
     b = poly.GetBounds()
-    sc = max([b[1] - b[0], b[3] - b[2], b[5] - b[4]]) / 50
+    sc = max([b[1] - b[0], b[3] - b[2], b[5] - b[4]]) / 50 *scale
     glyph.SetScaleFactor(sc)
     glyph.OrientOn()
     glyph.Update()
@@ -194,32 +140,6 @@ def normalLines(mesh, ratio=1, atCells=True):
     return glyphActor
 
 
-def extractLargestRegion(mesh):
-    """
-    Keep only the largest connected part of a mesh and discard all the smaller pieces.
-
-    .. hint:: |largestregion.py|_
-    """
-    conn = vtk.vtkConnectivityFilter()
-    conn.SetExtractionModeToLargestRegion()
-    conn.ScalarConnectivityOff()
-    if isinstance(mesh, vtk.vtkActor):
-        poly = mesh.GetMapper().GetInput()
-        prop = mesh.GetProperty()
-    elif isinstance(mesh, vtk.vtkPolyData):
-        poly = mesh
-        prop = None
-    conn.SetInputData(poly)
-    conn.Update()
-    epoly = conn.GetOutput()
-    eact = Mesh(epoly)
-    if prop:
-        pr = vtk.vtkProperty()
-        pr.DeepCopy(mesh.GetProperty())
-        eact.SetProperty(pr)
-    return eact
-
-
 def alignLandmarks(source, target, rigid=False):
     """
     Find best matching of source points towards target
@@ -262,11 +182,13 @@ def alignICP(source, target, iters=100, rigid=False):
 
          |align1| |align2|
     """
+    prop = None
     if isinstance(source, Mesh):
+        prop = vtk.vtkProperty()
+        prop.DeepCopy(source.GetProperty())
         source = source.polydata()
     if isinstance(target, Mesh):
         target = target.polydata()
-
     icp = vtk.vtkIterativeClosestPointTransform()
     icp.SetSource(source)
     icp.SetTarget(target)
@@ -281,6 +203,7 @@ def alignICP(source, target, iters=100, rigid=False):
     icpTransformFilter.Update()
     poly = icpTransformFilter.GetOutput()
     mesh = Mesh(poly)
+    if prop: mesh.SetProperty(prop)
 
     # mesh.info['transform'] = icp.GetLandmarkTransform() # not working!
     # do it manually...
@@ -522,8 +445,6 @@ def smoothMLS1D(mesh, f=0.2, radius=None, showNLines=0):
 
         |moving_least_squares1D| |skeletonize|
     """
-    if isinstance(mesh, Assembly):
-        mesh = mesh.getMeshes()[0]
     coords = mesh.points()
     ncoords = len(coords)
     Ncp = int(ncoords * f / 10)
@@ -579,8 +500,6 @@ def smoothMLS2D(mesh, f=0.2, radius=None, decimate=1, showNPlanes=0):
 
         |moving_least_squares2D| |recosurface|
     """
-    if isinstance(mesh, Assembly):
-        mesh = mesh.getMeshes()[0]
     coords = mesh.points()
     ncoords = len(coords)
     Ncp = int(ncoords * f / 100)
@@ -600,7 +519,7 @@ def smoothMLS2D(mesh, f=0.2, radius=None, decimate=1, showNPlanes=0):
     variances, newpts, acts = [], [], []
     pb = utils.ProgressBar(0, ncoords)
     for i, p in enumerate(coords):
-        pb.print("smoothing...")
+        pb.print("smoothing mesh ...")
         if i % decimate:
             continue
 
@@ -702,12 +621,12 @@ def recoSurface(pts, bins=256):
     """
     Surface reconstruction from a scattered cloud of points.
 
-    :param int bins: number of voxels in x, y and z.
+    :param int bins: number of voxels in x, y and z to control precision.
 
     |recosurface| |recosurface.py|_
     """
 
-    if isinstance(pts, vtk.vtkActor):
+    if isinstance(pts, Mesh):
         pts = pts.points()
     N = len(pts)
     if N < 50:
@@ -897,7 +816,8 @@ def _getinput(obj):
 
 def probePoints(vol, pts):
     """
-    Takes a ``Volume`` and probes its scalars at the specified points in space.
+    Takes a ``Volume`` (or any other vtk data set)
+    and probes its scalars at the specified points in space.
 
     Note that a mask is also output with valid/invalid points which can be accessed
     with `mesh.getPointArray()`.
@@ -936,7 +856,8 @@ def probePoints(vol, pts):
 
 def probeLine(vol, p1, p2, res=100):
     """
-    Takes a ``Volume`` and probes its scalars along a line defined by 2 points `p1` and `p2`.
+    Takes a ``Volume``  (or any other vtk data set)
+    and probes its scalars along a line defined by 2 points `p1` and `p2`.
 
     |probeLine| |probeLine.py|_
     """
@@ -958,7 +879,8 @@ def probeLine(vol, p1, p2, res=100):
 
 def probePlane(vol, origin=(0, 0, 0), normal=(1, 0, 0)):
     """
-    Takes a ``Volume`` or other vtk datasets and probes its scalars on a plane.
+    Takes a ``Volume`` (or any other vtk data set)
+    and probes its scalars on a plane defined by a point and a normal.
 
     |probePlane| |probePlane.py|_
     """
@@ -999,280 +921,11 @@ def resampleArrays(source, target, tol=None):
         return rs.GetOutput()
 
 
-def volumeOperation(volume1, operation, volume2=None):
-    """
-    Perform operations with ``Volume`` objects.
-
-    `volume2` can be a constant value.
-
-    Possible operations are: ``+``, ``-``, ``/``, ``1/x``, ``sin``, ``cos``, ``exp``, ``log``,
-    ``abs``, ``**2``, ``sqrt``, ``min``, ``max``, ``atan``, ``atan2``, ``median``,
-    ``mag``, ``dot``, ``gradient``, ``divergence``, ``laplacian``.
-
-    |volumeOperations| |volumeOperations.py|_
-    """
-    op = operation.lower()
-    image1 = _getinput(volume1)
-    image2 = _getinput(volume2)
-
-    if op in ["median"]:
-        mf = vtk.vtkImageMedian3D()
-        mf.SetInputData(image1)
-        mf.Update()
-        return Volume(mf.GetOutput())
-    elif op in ["mag"]:
-        mf = vtk.vtkImageMagnitude()
-        mf.SetInputData(image1)
-        mf.Update()
-        return Volume(mf.GetOutput())
-    elif op in ["dot", "dotproduct"]:
-        mf = vtk.vtkImageDotProduct()
-        mf.SetInput1Data(image1)
-        mf.SetInput2Data(image2)
-        mf.Update()
-        return Volume(mf.GetOutput())
-    elif op in ["grad", "gradient"]:
-        mf = vtk.vtkImageGradient()
-        mf.SetDimensionality(3)
-        mf.SetInputData(image1)
-        mf.Update()
-        return Volume(mf.GetOutput())
-    elif op in ["div", "divergence"]:
-        mf = vtk.vtkImageDivergence()
-        mf.SetInputData(image1)
-        mf.Update()
-        return Volume(mf.GetOutput())
-    elif op in ["laplacian"]:
-        mf = vtk.vtkImageLaplacian()
-        mf.SetDimensionality(3)
-        mf.SetInputData(image1)
-        mf.Update()
-        return Volume(mf.GetOutput())
-
-    mat = vtk.vtkImageMathematics()
-    mat.SetInput1Data(image1)
-    K = None
-    if image2:
-        if isinstance(image2, vtk.vtkImageData):
-            mat.SetInput2Data(image2)
-        else:  # assume image2 is a constant value
-            K = image2
-            mat.SetConstantK(K)
-            mat.SetConstantC(K)
-
-    if op in ["+", "add", "plus"]:
-        if K:
-            mat.SetOperationToAddConstant()
-        else:
-            mat.SetOperationToAdd()
-
-    elif op in ["-", "subtract", "minus"]:
-        if K:
-            mat.SetConstantC(-K)
-            mat.SetOperationToAddConstant()
-        else:
-            mat.SetOperationToSubtract()
-
-    elif op in ["*", "multiply", "times"]:
-        if K:
-            mat.SetOperationToMultiplyByK()
-        else:
-            mat.SetOperationToMultiply()
-
-    elif op in ["/", "divide"]:
-        if K:
-            mat.SetConstantK(1.0 / K)
-            mat.SetOperationToMultiplyByK()
-        else:
-            mat.SetOperationToDivide()
-
-    elif op in ["1/x", "invert"]:
-        mat.SetOperationToInvert()
-    elif op in ["sin"]:
-        mat.SetOperationToSin()
-    elif op in ["cos"]:
-        mat.SetOperationToCos()
-    elif op in ["exp"]:
-        mat.SetOperationToExp()
-    elif op in ["log"]:
-        mat.SetOperationToLog()
-    elif op in ["abs"]:
-        mat.SetOperationToAbsoluteValue()
-    elif op in ["**2", "square"]:
-        mat.SetOperationToSquare()
-    elif op in ["sqrt", "sqr"]:
-        mat.SetOperationToSquareRoot()
-    elif op in ["min"]:
-        mat.SetOperationToMin()
-    elif op in ["max"]:
-        mat.SetOperationToMax()
-    elif op in ["atan"]:
-        mat.SetOperationToATAN()
-    elif op in ["atan2"]:
-        mat.SetOperationToATAN2()
-    else:
-        colors.printc("~times Error in volumeOperation: unknown operation", operation, c=1)
-        raise RuntimeError()
-    mat.Update()
-    return Volume(mat.GetOutput())
-
-
-def thinPlateSpline(mesh, sourcePts, targetPts, userFunctions=(None, None), sigma=1):
-    """
-    `Thin Plate Spline` transformations describe a nonlinear warp transform defined by a set
-    of source and target landmarks. Any point on the mesh close to a source landmark will
-    be moved to a place close to the corresponding target landmark.
-    The points in between are interpolated smoothly using Bookstein's Thin Plate Spline algorithm.
-
-    Transformation object can be retrieved with ``mesh.getTransform()``.
-
-    :param userFunctions: You may supply both the function and its derivative with respect to r.
-
-    .. hint:: Examples: |thinplate.py|_ |thinplate_grid.py|_ |thinplate_morphing.py|_
-        |interpolateField.py|_ |thinplate_morphing_2d.py|_
-
-        |thinplate| |thinplate_grid| |thinplate_morphing| |interpolateField| |thinplate_morphing_2d|
-    """
-    ns = len(sourcePts)
-    ptsou = vtk.vtkPoints()
-    ptsou.SetNumberOfPoints(ns)
-    for i in range(ns):
-        ptsou.SetPoint(i, sourcePts[i])
-
-    nt = len(targetPts)
-    if ns != nt:
-        colors.printc("~times thinPlateSpline Error: #source != #target points", ns, nt, c=1)
-        raise RuntimeError()
-
-    pttar = vtk.vtkPoints()
-    pttar.SetNumberOfPoints(nt)
-    for i in range(ns):
-        pttar.SetPoint(i, targetPts[i])
-
-    transform = vtk.vtkThinPlateSplineTransform()
-    transform.SetBasisToR()
-    if userFunctions[0]:
-        transform.SetBasisFunction(userFunctions[0])
-        transform.SetBasisDerivative(userFunctions[1])
-    transform.SetSigma(sigma)
-    transform.SetSourceLandmarks(ptsou)
-    transform.SetTargetLandmarks(pttar)
-
-    tfa = transformFilter(mesh.polydata(), transform)
-    tfa.info["transform"] = transform
-    return tfa
-
-
-def warpMeshToPoint(mesh, point, factor=0.1, absolute=True):
-    """
-    Modify the mesh coordinates by moving the vertices towards a specified point.
-
-    :param float factor: value to scale displacement.
-    :param list point: the position to warp towards.
-    :param bool absolute: turning on causes scale factor of the new position
-        to be one unit away from point.
-
-    :Example:
-        .. code-block:: python
-
-            from vtkplotter import *
-            s = Cylinder(height=3).wireframe(1)
-            pt = [4,0,0]
-            w = warpMeshToPoint(s, pt, factor=0.5).wireframe(0)
-            show(w,s, Point(pt), axes=1, bg='w')
-
-        |warpto|
-    """
-    warpTo = vtk.vtkWarpTo()
-    warpTo.SetInputData(mesh.polydata())
-    warpTo.SetPosition(point)
-    warpTo.SetScaleFactor(factor)
-    warpTo.SetAbsolute(absolute)
-    warpTo.Update()
-    prop = vtk.vtkProperty()
-    prop.DeepCopy(mesh.GetProperty())
-    a = Mesh(warpTo.GetOutput())
-    a.SetProperty(prop)
-    return a
-
-
-def transformFilter(mesh, transformation):
-    """
-    Transform a ``Mesh`` and return a new object.
-    """
-    tf = vtk.vtkTransformPolyDataFilter()
-    tf.SetTransform(transformation)
-    prop = None
-    if isinstance(mesh, vtk.vtkPolyData):
-        tf.SetInputData(mesh)
-    else:
-        tf.SetInputData(mesh.polydata())
-        prop = vtk.vtkProperty()
-        prop.DeepCopy(mesh.GetProperty())
-    tf.Update()
-
-    tfa = Mesh(tf.GetOutput())
-    if prop:
-        tfa.SetProperty(prop)
-    return tfa
-
-
-def meshQuality(mesh, measure=6):
-    """
-    Calculate functions of quality of the elements of a triangular mesh.
-    See class `vtkMeshQuality <https://vtk.org/doc/nightly/html/classvtkMeshQuality.html>`_
-    for explanation.
-
-    :param int measure: type of estimator
-
-        - EDGE_RATIO, 0
-        - ASPECT_RATIO, 1
-        - RADIUS_RATIO, 2
-        - ASPECT_FROBENIUS, 3
-        - MED_ASPECT_FROBENIUS, 4
-        - MAX_ASPECT_FROBENIUS, 5
-        - MIN_ANGLE, 6
-        - COLLAPSE_RATIO, 7
-        - MAX_ANGLE, 8
-        - CONDITION, 9
-        - SCALED_JACOBIAN, 10
-        - SHEAR, 11
-        - RELATIVE_SIZE_SQUARED, 12
-        - SHAPE, 13
-        - SHAPE_AND_SIZE, 14
-        - DISTORTION, 15
-        - MAX_EDGE_RATIO, 16
-        - SKEW, 17
-        - TAPER, 18
-        - VOLUME, 19
-        - STRETCH, 20
-        - DIAGONAL, 21
-        - DIMENSION, 22
-        - ODDY, 23
-        - SHEAR_AND_SIZE, 24
-        - JACOBIAN, 25
-        - WARPAGE, 26
-        - ASPECT_GAMMA, 27
-        - AREA, 28
-        - ASPECT_BETA, 29
-
-    |meshquality| |meshquality.py|_
-    """
-
-    poly = mesh.GetMapper().GetInput()
-
-    qf = vtk.vtkMeshQuality()
-    qf.SetInputData(poly)
-    qf.SetTriangleQualityMeasure(measure)
-    qf.SaveCellQualityOn()
-    qf.Update()
-
-    pd = vtk.vtkPolyData()
-    pd.ShallowCopy(qf.GetOutput())
-
-    qmesh = Mesh(pd)
-    qmesh.mapper().SetScalarRange(pd.GetScalarRange())
-    return qmesh
+def thinPlateSpline(*args, **kwargs):
+    """Obsolete! Use mesh.thinPlateSpline() and mesh.applyTransform() instead."""
+    colors.printc("WARNING: thinPlateSpline(mesh) is obsolete!", c=1)
+    colors.printc("       : Use mesh.thinPlateSpline() and mesh.applyTransform() instead.", c=1)
+    raise RuntimeError()
 
 
 def connectedPoints(mesh, radius, mode=0, regions=(), vrange=(0,1), seeds=(), angle=0):
@@ -1354,42 +1007,6 @@ def connectedPoints(mesh, radius, mode=0, regions=(), vrange=(0,1), seeds=(), an
     return Mesh(cpf.GetOutput())
 
 
-def splitByConnectivity(mesh, maxdepth=1000):
-    """
-    Split a mesh by connectivity and order the pieces by increasing area.
-
-    :param int maxdepth: only consider this number of mesh parts.
-
-    |splitmesh| |splitmesh.py|_
-    """
-    mesh.addIDs()
-    pd = mesh.polydata()
-    cf = vtk.vtkConnectivityFilter()
-    cf.SetInputData(pd)
-    cf.SetExtractionModeToAllRegions()
-    cf.ColorRegionsOn()
-    cf.Update()
-    cpd = cf.GetOutput()
-    a = Mesh(cpd)
-    alist = []
-
-    for t in range(max(a.getPointArray("RegionId")) - 1):
-        if t == maxdepth:
-            break
-        suba = a.clone().threshold("RegionId", t - 0.1, t + 0.1)
-        area = suba.area()
-        alist.append([suba, area])
-
-    alist.sort(key=lambda x: x[1])
-    alist.reverse()
-    blist = []
-    for i, l in enumerate(alist):
-        l[0].color(i + 1).phong()
-        l[0].mapper().ScalarVisibilityOff()
-        blist.append(l[0])
-    return blist
-
-
 def pointSampler(mesh, distance=None):
     """
     Algorithm to generate points the specified distance apart.
@@ -1889,7 +1506,8 @@ def streamLines(domain, probe,
                 lw=None,
     ):
     """
-    Integrate a vector field to generate streamlines.
+    Integrate a vector field on a domain (a Mesh or other vtk datasets types)
+    to generate streamlines.
 
     The integration is performed using a specified integrator (Runge-Kutta).
     The length of a streamline is governed by specifying a maximum value either
@@ -2114,61 +1732,6 @@ def densifyCloud(mesh, targetDistance, closestN=6, radius=0, maxIter=None, maxN=
     return shapes.Points(pts, c=None).pointSize(3)
 
 
-def frequencyPassFilter(volume, lowcutoff=None, highcutoff=None, order=1):
-    """
-    Low-pass and high-pass filtering become trivial in the frequency domain.
-    A portion of the pixels/voxels are simply masked or attenuated.
-    This function applies a high pass Butterworth filter that attenuates the frequency domain
-    image with the function
-
-    |G_Of_Omega|
-
-    The gradual attenuation of the filter is important.
-    A simple high-pass filter would simply mask a set of pixels in the frequency domain,
-    but the abrupt transition would cause a ringing effect in the spatial domain.
-
-    :param list lowcutoff:  the cutoff frequencies for x, y and z
-    :param list highcutoff: the cutoff frequencies for x, y and z
-    :param int order: order determines sharpness of the cutoff curve
-
-    Check out also this example:
-
-    |idealpass|
-    """
-    #https://lorensen.github.io/VTKExamples/site/Cxx/ImageProcessing/IdealHighPass
-    img = _getinput(volume)
-    fft = vtk.vtkImageFFT()
-    fft.SetInputData(img)
-    fft.Update()
-    out = fft.GetOutput()
-
-    if highcutoff:
-        butterworthLowPass = vtk.vtkImageButterworthLowPass()
-        butterworthLowPass.SetInputData(out)
-        butterworthLowPass.SetCutOff(highcutoff) # actually inverted..(?)
-        butterworthLowPass.SetOrder(order)
-        butterworthLowPass.Update()
-        out = butterworthLowPass.GetOutput()
-
-    if lowcutoff:
-        butterworthHighPass = vtk.vtkImageButterworthHighPass()
-        butterworthHighPass.SetInputData(out)
-        butterworthHighPass.SetCutOff(lowcutoff)
-        butterworthHighPass.SetOrder(order)
-        butterworthHighPass.Update()
-        out = butterworthHighPass.GetOutput()
-
-    butterworthRfft = vtk.vtkImageRFFT()
-    butterworthRfft.SetInputData(out)
-    butterworthRfft.Update()
-
-    butterworthReal = vtk.vtkImageExtractComponents()
-    butterworthReal.SetInputData(butterworthRfft.GetOutput())
-    butterworthReal.SetComponents(0)
-    butterworthReal.Update()
-    return Volume(butterworthReal.GetOutput())
-
-
 def implicitModeller(mesh, distance=0.05, res=(110,40,20), bounds=(), maxdist=None, outer=True):
     """Finds the surface at the specified distance from the input one"""
 
@@ -2189,8 +1752,7 @@ def implicitModeller(mesh, distance=0.05, res=(110,40,20), bounds=(), maxdist=No
     contour.Update()
     poly = contour.GetOutput()
     if outer:
-        poly = extractLargestRegion(poly)
-
+        return Mesh(poly).extractLargestRegion().c('lb')
     return Mesh(poly, c='lb')
 
 
@@ -2261,43 +1823,6 @@ def volumeFromMesh(mesh, bounds=None, dims=(20,20,20), signed=True, negate=False
     return Volume(img)
 
 
-def computeNormalsWithPCA(mesh, n=20, orientationPoint=None, negate=False):
-    """Generate point normals using PCA (principal component analysis).
-    Basically this estimates a local tangent plane around each sample point p
-    by considering a small neighborhood of points around p, and fitting a plane
-    to the neighborhood (via PCA).
-
-    :param int n: neighborhood size to calculate the normal
-    :param list orientationPoint: adjust the +/- sign of the normals so that
-        the normals all point towards a specified point. If None, perform a traversal
-        of the point cloud and flip neighboring normals so that they are mutually consistent.
-
-    :param bool negate: flip all normals
-    """
-    poly = mesh.polydata()
-    pcan = vtk.vtkPCANormalEstimation()
-    pcan.SetInputData(poly)
-    pcan.SetSampleSize(n)
-
-    if orientationPoint is not None:
-        pcan.SetNormalOrientationToPoint()
-        pcan.SetOrientationPoint(orientationPoint)
-    else:
-        pcan.SetNormalOrientationToGraphTraversal()
-
-    if negate:
-        pcan.FlipNormalsOn()
-
-    pcan.Update()
-    out = pcan.GetOutput()
-    vnorm = out.GetPointData().GetNormals()
-
-    newact = mesh.clone()
-    newact.polydata().GetPointData().SetNormals(vnorm)
-    newact.polydata().GetPointData().Modified()
-    return newact
-
-
 def pointDensity(mesh, dims=(30,30,30), bounds=None, radius=None, computeGradient=False):
     """Generate a density field from a point cloud. Output is a ``Volume``.
     The local neighborhood is specified as a `radius` around each sample position (each voxel).
@@ -2321,92 +1846,9 @@ def pointDensity(mesh, dims=(30,30,30), bounds=None, radius=None, computeGradien
     pdf.Update()
     img = pdf.GetOutput()
     vol = Volume(img)
-    return volumeOperation(vol, '/', img.GetScalarRange()[1])
+    return vol.operation('/', img.GetScalarRange()[1])
 
 
-def erodeVolume(vol, neighbours=(2,2,2)):
-    """Replace a voxel with the minimum over an ellipsoidal neighborhood of voxels.
-    If `neighbours` of an axis is 1, no processing is done on that axis.
-
-    See example script: |erode_dilate.py|_
-    """
-    img = _getinput(vol)
-    ver = vtk.vtkImageContinuousErode3D()
-    ver.SetInputData(img)
-    ver.SetKernelSize(neighbours[0], neighbours[1], neighbours[2])
-    ver.Update()
-    return Volume(ver.GetOutput())
-
-
-def dilateVolume(vol, neighbours=(2,2,2)):
-    """Replace a voxel with the maximum over an ellipsoidal neighborhood of voxels.
-    If `neighbours` of an axis is 1, no processing is done on that axis.
-
-    See example script: |erode_dilate.py|_
-    """
-    img = _getinput(vol)
-    ver = vtk.vtkImageContinuousDilate3D()
-    ver.SetInputData(img)
-    ver.SetKernelSize(neighbours[0], neighbours[1], neighbours[2])
-    ver.Update()
-    return Volume(ver.GetOutput())
-
-
-def euclideanDistanceVolume(vol, anisotropy=False, maxDistance=None):
-    """Implementation of the Euclidean DT (Distance Transform) using Saito's algorithm.
-    The distance map produced contains the square of the Euclidean distance values.
-    The algorithm has a O(n^(D+1)) complexity over nxnx...xn images in D dimensions.
-
-    Check out also: https://en.wikipedia.org/wiki/Distance_transform
-
-    :param bool anisotropy: used to define whether Spacing should be used
-        in the computation of the distances.
-    :param float maxDistance: any distance bigger than maxDistance will not be
-        computed but set to this specified value instead.
-
-    See example script: |euclDist.py|_
-    """
-    img = _getinput(vol)
-    euv = vtk.vtkImageEuclideanDistance()
-    euv.SetInputData(img)
-    euv.SetConsiderAnisotropy(anisotropy)
-    if maxDistance is not None:
-        euv.InitializeOn()
-        euv.SetMaximumDistance(maxDistance)
-    euv.SetAlgorithmToSaito()
-    euv.Update()
-    return Volume(euv.GetOutput())
-
-
-def volumeToPoints(vol):
-    """Extract all image voxels as points.
-    This function takes an input ``Volume`` and creates an ``Mesh``
-    that contains the points and the point attributes.
-
-    See example script: |vol2points.py|_
-    """
-    img = _getinput(vol)
-    v2p = vtk.vtkImageToPoints()
-    v2p.SetInputData(img)
-    v2p.Update()
-    return Mesh(v2p.GetOutput())
-
-
-def volumeCorrelation(vol1, vol2, dim=2):
-    """Find the correlation between two volumetric data sets.
-    Keyword `dim` determines whether the correlation will be 3D, 2D or 1D.
-    The default is a 2D Correlation.
-    The output size will match the size of the first input.
-    The second input is considered the correlation kernel.
-    """
-    img1 = _getinput(vol1)
-    img2 = _getinput(vol2)
-    imc = vtk.vtkImageCorrelation()
-    imc.SetInput1Data(img1)
-    imc.SetInput2Data(img2)
-    imc.SetDimensionality(dim)
-    imc.Update()
-    return Volume(imc.GetOutput())
 
 
 
