@@ -20,11 +20,13 @@ from vtkplotter.mesh import Mesh
 from vtkplotter.vtkio import load, screenshot, Video, exportWindow
 
 import vtkplotter.shapes as shapes
-from vtkplotter.shapes import Text, Latex
+from vtkplotter.shapes import Text, Text2D, Latex
 
 from vtkplotter.plotter import show, clear, Plotter
 from vtkplotter.plotter import closeWindow, closePlotter, interactive
 
+from vtkplotter.pyplot import histogram
+
 # Install fenics with commands (e.g. in Anaconda3):
 #         conda install -c conda-forge fenics
 #         pip install vtkplotter
@@ -103,10 +105,7 @@ Image Gallery
 
 __all__ = [
     "plot",
-    "MeshActor",
-    "MeshPoints",
-    "MeshLines",
-    "MeshArrows",
+    "histogram",
     "load",
     "show",
     "clear",
@@ -115,6 +114,7 @@ __all__ = [
     "Plotter",
     "ProgressBar",
     "Text",
+    "Text2D",
     "Latex",
     "datadir",
     "screenshot",
@@ -340,7 +340,18 @@ def plot(*inputobj, **options):
         - lw, (float) - isoline width
         - z, (float) - add to the isoline z coordinate to make them more visible
 
-    :param float warpZfactor: elevate z-axis by scalar value (useful for 2D geometries)
+
+    :param dict streamlines: dictionary of streamlines properties
+    
+        - probes, (list, None) - custom list of points to use as seeds
+        - tol, (float) - tolerance to reduce the number of seed points used in mesh
+        - lw, (float) - line width of the streamline
+        - direction, (str) - direction of integration ('forward', 'backward' or 'both')
+        - maxPropagation, (float) - max propagation of the streamline
+        - scalarRange, (list) - scalar range of coloring
+
+
+   :param float warpZfactor: elevate z-axis by scalar value (useful for 2D geometries)
     :param float warpYfactor: elevate z-axis by scalar value (useful for 1D geometries)
 
     :param list scaleMeshFactors: rescale mesh by these factors [1,1,1]
@@ -382,9 +393,8 @@ def plot(*inputobj, **options):
     Axes type-1 can be fully customized by passing a dictionary ``axes=dict()`` where:
 
         - `xtitle`,            ['x'], x-axis title text.
-        - `ytitle`,            ['y'], y-axis title text.
-        - `ztitle`,            ['z'], z-axis title text.
-        - `numberOfDivisions`, [automatic], approximate number of divisions on the longest axis
+        - `xrange`,           [None], x-axis range in format (xmin, ymin), default is automatic.
+        - `numberOfDivisions`,[None], approximate number of divisions on the longest axis
         - `axesLineWidth`,       [1], width of the axes lines
         - `gridLineWidth`,       [1], width of the grid lines
         - `reorientShortTitle`, [True], titles shorter than 2 letter are placed horizontally
@@ -473,10 +483,14 @@ def plot(*inputobj, **options):
     :param int interactorStyle: change the style of muose interaction of the scene
     :param bool q: exit python session after returning.
     """
-
-    if len(inputobj) == 0:
-        return interactive()
-
+    if len(inputobj)==0:
+        interactive()
+        return
+    
+    if 'numpy' in str(type(inputobj[0])):
+        from vtkplotter.pyplot import plot as pyplot_plot
+        return pyplot_plot(*inputobj, **options)
+    
     mesh, u = _inputsort(inputobj)
 
     mode = options.pop("mode", 'mesh')
@@ -508,11 +522,13 @@ def plot(*inputobj, **options):
     text = options.pop("text", None)
     style = options.pop("style", 'vtk')
     isolns = options.pop("isolines", dict())
+    streamlines = options.pop("streamlines", dict())
     warpZfactor = options.pop("warpZfactor", None)
     warpYfactor = options.pop("warpYfactor", None)
     lighting = options.pop("lighting", None)
     exterior = options.pop("exterior", False)
     fast = options.pop("fast", False)
+    returnActorsNoShow = options.pop("returnActorsNoShow", False)
 
     # refresh axes titles for axes type = 8 (vtkCubeAxesActor)
     settings.xtitle = options.pop("xtitle", 'x')
@@ -634,6 +650,7 @@ def plot(*inputobj, **options):
     if mesh and ('mesh' in mode or 'color' in mode or 'displace' in mode):
 
         actor = MeshActor(u, mesh, exterior=exterior, fast=fast)
+
         actor.wireframe(wire)
         actor.scale(scaleMeshFactors)
         if lighting:
@@ -710,6 +727,12 @@ def plot(*inputobj, **options):
         actors.append(actor)
 
 
+    #################################################################
+    if 'streamline' in mode:
+        mode = mode.replace('streamline','')
+        str_act = MeshStreamLines(u, **streamlines)
+        actors.append(str_act)
+
     #################################################################
     if 'arrow' in mode or 'line' in mode:
         if 'arrow' in mode:
@@ -732,7 +755,6 @@ def plot(*inputobj, **options):
     if 'tensor' in mode:
         pass #todo
 
-
     #################################################################
     for ob in inputobj:
         inputtype = str(type(ob))
@@ -740,7 +762,7 @@ def plot(*inputobj, **options):
            actors.append(ob)
 
     if text:
-        textact = Text(text, font=font)
+        textact = Text2D(text, font=font)
         actors.append(textact)
 
     if 'at' in options.keys() and 'interactive' not in options.keys():
@@ -759,6 +781,9 @@ def plot(*inputobj, **options):
     if len(actors)==0:
          print('Warning: no objects to show, check mode in plot(mode="...")')
 
+    if returnActorsNoShow:
+        return actors
+
     return show(actors, **options)
 
 
@@ -768,7 +793,7 @@ class MeshActor(Mesh):
 
     def __init__(self, *inputobj, **options):
 
-        c = options.pop("c", "gold")
+        c = options.pop("c", None)
         alpha = options.pop("alpha", 1)
         exterior = options.pop("exterior", False)
         fast = options.pop("fast", False)
@@ -802,7 +827,7 @@ class MeshActor(Mesh):
         self.u = u  # holds a dolfin function_data
         # holds the actual values of u on the mesh
         self.u_values = _compute_uvalues(u, mesh)
-        #self.addPointScalars(self.u_values, "u_values")
+        #self.addPointArray(self.u_values, "u_values")
 
 
     def move(self, u=None, deltas=None):
@@ -963,3 +988,70 @@ def MeshArrows(*inputobj, **options):
     actor.u = u
     actor.u_values = u_values
     return actor
+
+
+def MeshStreamLines(*inputobj, **options):
+    """
+    Build streamplot.
+    """
+    from vtkplotter.analysis import streamLines
+
+    print('Building streamlines...')
+
+    tol = options.pop('tol', 0.02)
+    lw = options.pop('lw', 2)
+    direction = options.pop('direction', 'forward')
+    maxPropagation = options.pop('maxPropagation', None)
+    scalarRange = options.pop('scalarRange', None)
+    probes = options.pop('probes', None)
+
+    tubes = options.pop('tubes', dict()) # todo
+    maxRadiusFactor = options.pop('maxRadiusFactor', 1)
+    varyRadius = options.pop('varyRadius', 1)
+
+    mesh, u = _inputsort(inputobj)
+    if not mesh:
+        return None
+
+    u_values = _compute_uvalues(u, mesh)
+    if not utils.isSequence(u_values[0]):
+        printc("~times Error: cannot show Arrows for 1D scalar values!", c=1)
+        raise RuntimeError()
+    if u_values.shape[1] == 2:  # u_values is 2D
+        u_values = np.insert(u_values, 2, 0, axis=1)  # make it 3d
+
+    meshact = MeshActor(u)
+    meshact.addPointVectors(u_values, 'u_values')
+
+    if utils.isSequence(probes):
+        pass # it's already it
+    elif tol:
+        print('decimating mesh points to use them as seeds...')
+        probes = meshact.clone().clean(tol).points()
+    else:
+        probes = meshact.points()
+    if len(probes)>500:
+        printc('Probing domain with n =', len(probes), 'points')
+        printc(' ..this may take time (or choose a larger tol value)')
+
+    if lw:
+        tubes = dict()
+    else:
+        tubes['varyRadius'] = varyRadius
+        tubes['maxRadiusFactor'] = maxRadiusFactor
+
+    str_lns = streamLines(meshact, probes,
+                          direction=direction,
+                          maxPropagation=maxPropagation,
+                          tubes=tubes,
+                          scalarRange=scalarRange,
+                          activeVectors='u_values')
+
+    if lw:
+        str_lns.lw(lw)
+
+    return str_lns
+
+
+
+
