@@ -2,7 +2,7 @@ from __future__ import division, print_function
 import vtk
 import numpy as np
 from vtkplotter import settings
-from vtk.util.numpy_support import numpy_to_vtk, vtk_to_numpy
+from vtk.util.numpy_support import numpy_to_vtk
 import vtkplotter.utils as utils
 from vtkplotter.colors import printc, getColor, colorMap, _mapscales
 from vtkplotter.mesh import Mesh
@@ -38,7 +38,6 @@ __all__ = [
     "Star",
     "Sphere",
     "Spheres",
-    "SphericGrid",
     "Earth",
     "Ellipsoid",
     "Grid",
@@ -54,6 +53,7 @@ __all__ = [
     "Paraboloid",
     "Hyperboloid",
     "Text",
+    "Text2D",
     "Latex",
     "Glyph",
     "Tensors",
@@ -62,21 +62,22 @@ __all__ = [
 
 
 ########################################################################
-def Marker(symbol, c='lb', alpha=1, s=0.1, filled=True):
+def Marker(symbol, pos=(0, 0, 0), c='lb', alpha=1, s=0.1, filled=True):
     """
     Generate a marker shape.
     Can be used in association with ``Glyph``.
     """
     if isinstance(symbol, int):
         symbs = ['.', 'p','*','h','D','d','o','v','^','>','<','s', 'x', 'a']
-        symbol = symbs[s]
+        symbol = symbol % 14
+        symbol = symbs[symbol]
 
     if symbol == '.':
         mesh = Polygon(nsides=24, r=s*0.75)
     elif symbol == 'p':
         mesh = Polygon(nsides=5, r=s)
     elif symbol == '*':
-        mesh = Star(r1=0.7*s, r2=s, line=not filled)
+        mesh = Star(r1=0.65*s*1.1, r2=s*1.1, line=not filled)
     elif symbol == 'h':
         mesh = Polygon(nsides=6, r=s)
     elif symbol == 'D':
@@ -102,7 +103,11 @@ def Marker(symbol, c='lb', alpha=1, s=0.1, filled=True):
         mesh = Text('*', pos=(0,0,0), s=s*3, justify='center', depth=0)
     else:
         mesh = Text(symbol, pos=(0,0,0), s=s*2, justify='center', depth=0)
+    settings.collectable_actors.pop()
     mesh.flat().lighting('ambient').wireframe(not filled).c(c).alpha(alpha)
+    if len(pos) == 2:
+        pos = (pos[0], pos[1], 0)
+    mesh.SetPosition(pos)
     mesh.name = "Marker"
     return mesh
 
@@ -134,7 +139,7 @@ class Points(Mesh):
 
     |lorenz|
     """
-    def __init__(self, plist, r=5, c="gold", alpha=1):
+    def __init__(self, plist, r=5, c=(.3,.3,.3), alpha=1):
 
         ################ interpret user input format:
         if isinstance(plist, Mesh):
@@ -162,7 +167,7 @@ class Points(Mesh):
 
                 n = len(plist)
                 if n != len(cols):
-                    printc("~times mismatch in Points() colors", n, len(cols), c=1)
+                    printc("Mismatch in Points() colors", n, len(cols), c=1)
                     raise RuntimeError()
                 src = vtk.vtkPointSource()
                 src.SetNumberOfPoints(n)
@@ -178,7 +183,7 @@ class Points(Mesh):
                 ucols.SetName("pointsRGBA")
                 if utils.isSequence(alpha):
                     if len(alpha) != n:
-                        printc("~times mismatch in Points() alphas", n, len(alpha), c=1)
+                        printc("Mismatch in Points() alphas", n, len(alpha), c=1)
                         raise RuntimeError()
                     alphas = alpha
                     alpha = 1
@@ -270,6 +275,7 @@ class Glyph(Mesh):
         if utils.isSequence(mesh):
             # create a cloud of points
             mesh = Points(mesh)
+            settings.collectable_actors.pop()
 
         if tol:
             mesh = mesh.clone().clean(tol)
@@ -465,9 +471,13 @@ class Line(Mesh):
             ppoints = vtk.vtkPoints()  # Generate the polyline
             dim = len((p0[0]))
             if dim == 2:
+                firstpt = np.array([p0[0][0], p0[0][1], 0])
+                lastpt = np.array([p0[-1][0], p0[-1][0], 0])
                 for i, p in enumerate(p0):
                     ppoints.InsertPoint(i, p[0], p[1], 0)
             else:
+                firstpt = np.array(p0[0])
+                lastpt = np.array(p0[-1])
                 ppoints.SetData(numpy_to_vtk(np.ascontiguousarray(p0), deep=True))
             lines = vtk.vtkCellArray()  # Create the polyline.
             lines.InsertNextCell(len(p0))
@@ -484,14 +494,17 @@ class Line(Mesh):
                 lineSource.SetResolution(res)
             lineSource.Update()
             poly = lineSource.GetOutput()
+            firstpt = np.array(p0)
+            lastpt = np.array(p1)
 
         Mesh.__init__(self, poly, c, alpha)
         self.lw(lw)
         if dotted:
             self.GetProperty().SetLineStipplePattern(0xF0F0)
             self.GetProperty().SetLineStippleRepeatFactor(1)
-        self.base = np.array(p0)
-        self.top  = np.array(p1)
+        self.base = firstpt
+        self.top  = lastpt
+        #self.SetOrigin((firstpt+lastpt)/2)
         settings.collectable_actors.append(self)
         self.name = "Line"
 
@@ -502,13 +515,13 @@ class DashedLine(Mesh):
     If `p0` is a list of points returns the line connecting them.
     A 2D set of coords can also be passed as p0=[x..], p1=[y..].
 
-    :param float spacing: physical size of the dash.
+    :param float spacing: relative size of the dash.
     :param c: color name, number, or list of [R,G,B] colors.
     :type c: int, str, list
     :param float alpha: transparency in range [0,1].
     :param lw: line width.
     """
-    def __init__(self, p0, p1=None, spacing=None, c="red", alpha=1, lw=1):
+    def __init__(self, p0, p1=None, spacing=0.1, c="red", alpha=1, lw=2):
 
         if isinstance(p0, vtk.vtkActor): p0 = p0.GetPosition()
         if isinstance(p1, vtk.vtkActor): p1 = p1.GetPosition()
@@ -520,46 +533,67 @@ class DashedLine(Mesh):
                 p0 = np.stack((p0, p1), axis=1)
                 p1 = None
 
+        if p1 is not None: # assume passing p0=[x,y]
+            if len(p0) == 2 and not utils.isSequence(p0[0]):
+                p0 = (p0[0], p0[1], 0)
+            if len(p1) == 2 and not utils.isSequence(p1[0]):
+                p1 = (p1[0], p1[1], 0)
+
         # detect if user is passing a list of points:
         if utils.isSequence(p0[0]):
            listp = p0
         else:  # or just 2 points to link
             listp = [p0, p1]
 
-        if not spacing:
-            spacing = np.linalg.norm(np.array(listp[-1]) - listp[0])/50
+        listp = np.array(listp)
+        if listp.shape[1]==2:
+            listp = np.c_[listp, np.zeros(listp.shape[0])]
 
-        polylns = vtk.vtkAppendPolyData()
-        for ipt in range(1, len(listp)):
-            p0 = np.array(listp[ipt-1])
-            p1 = np.array(listp[ipt])
+        xmn = np.min(listp, axis=0)
+        xmx = np.max(listp, axis=0)
+        dlen = np.linalg.norm(xmx-xmn)*spacing/10
+        if not dlen:
+            printc("Error in DashedLine: zero dash length.", c=1)
+            Mesh.__init__(self, vtk.vtkPolyData(), c, alpha)
+            return
+
+        qs = []
+        for ipt in range(len(listp)-1):
+            p0 = listp[ipt]
+            p1 = listp[ipt+1]
             v = p1-p0
-            n1 = int(np.linalg.norm(v)/spacing)
+            vdist = np.linalg.norm(v)
+            n1 = int(vdist/dlen)
             if not n1: continue
 
-            for i in range(1, n1+2):
-                if (i-1)/n1>1:
-                    continue
+            res = 0
+            for i in range(n1+2):
+                ist = (i-0.5)/n1
+                if ist<0: ist=0
+                qi = p0 + v * (ist - res/vdist)
+                if ist>1:
+                    qi = p1
+                    res = np.linalg.norm(qi-p1)
+                    qs.append(qi)
+                    break
+                qs.append(qi)
 
-                if i%2:
-                    q0 = p0 + (i-1)/n1*v
-                    if i/n1>1:
-                        q1 = p1
-                    else:
-                        q1 = p0 + i/n1*v
-                    lineSource = vtk.vtkLineSource()
-                    lineSource.SetPoint1(q0)
-                    lineSource.SetPoint2(q1)
-                    lineSource.Update()
-                    polylns.AddInputData(lineSource.GetOutput())
-
-            polylns.Update()
-            poly = polylns.GetOutput()
+        polylns = vtk.vtkAppendPolyData()
+        for i,q1 in enumerate(qs):
+            if not i%2: continue
+            q0 = qs[i-1]
+            lineSource = vtk.vtkLineSource()
+            lineSource.SetPoint1(q0)
+            lineSource.SetPoint2(q1)
+            lineSource.Update()
+            polylns.AddInputData(lineSource.GetOutput())
+        polylns.Update()
+        poly = polylns.GetOutput()
 
         Mesh.__init__(self, poly, c, alpha)
         self.lw(lw)
-        self.base = np.array(p0)
-        self.top  = np.array(p1)
+        self.base = listp[0]
+        self.top  = listp[-1]
         settings.collectable_actors.append(self)
         self.name = "DashedLine"
 
@@ -650,7 +684,7 @@ class Spline(Mesh):
             lines.InsertCellPoint(i)
         profileData.SetPoints(ppoints)
         profileData.SetLines(lines)
-        Mesh.__init__(self, profileData)
+        Mesh.__init__(self, profileData, c='k')
         self.GetProperty().SetLineWidth(s)
         self.base = np.array(points[0])
         self.top = np.array(points[-1])
@@ -949,8 +983,8 @@ class Arrow2D(Mesh):
     """
     def __init__(self, startPoint, endPoint,
                  shaftLength=0.8,
-                 shaftWidth=0.09,
-                 headLength=None,
+                 shaftWidth=0.05,
+                 headLength=0.25,
                  headWidth=0.2,
                  fill=True,
                  c="r",
@@ -1002,6 +1036,7 @@ class Arrow2D(Mesh):
         tf.Update()
 
         Mesh.__init__(self, tf.GetOutput(), c, alpha)
+        self.SetPosition(startPoint)
         self.flat().lighting('ambient')
         self.DragableOff()
         self.PickableOff()
@@ -1043,7 +1078,7 @@ def Arrows2D(startPoints, endPoints=None,
             arrs2d = Arrows2D(g1, g2, c='jet')
             arrs2d.show(axes=1, bg='white')
 
-        |quiverPlot|
+        |quiver|
     """
     if isinstance(startPoints, Mesh): startPoints = startPoints.points()
     if isinstance(endPoints,   Mesh): endPoints   = endPoints.points()
@@ -1113,30 +1148,39 @@ class Polygon(Mesh):
     |Polygon|
     """
     def __init__(self, pos=(0, 0, 0), nsides=6, r=1, c="coral", alpha=1):
-        ps = vtk.vtkRegularPolygonSource()
-        ps.SetNumberOfSides(nsides)
-        ps.SetRadius(r)
-        ps.Update()
-        Mesh.__init__(self, ps.GetOutput(), c, alpha)
+
+        if len(pos) == 2:
+            pos = (pos[0], pos[1], 0)
+
+        t = np.linspace(np.pi/2, 5/2*np.pi, num=nsides, endpoint=False)
+        x, y = utils.pol2cart(np.ones_like(t)*r, t)
+        faces = [list(range(nsides))]
+        Mesh.__init__(self, [np.c_[x,y], faces], c, alpha)
+
+        #ps = vtk.vtkRegularPolygonSource() # bugged!
+        #ps.SetNumberOfSides(nsides)
+        #ps.SetRadius(r)
+        #ps.Update()
+        #Mesh.__init__(self, ps.GetOutput(), c, alpha)
         self.SetPosition(pos)
         settings.collectable_actors.append(self)
-        self.name = "Polygon"
+        self.name = "Polygon " + str(nsides)
 
 
 class Circle(Polygon):
     """
     Build a Circle of radius `r`.
     """
-    def __init__(self, pos=(0,0,0), r=1, fill=False, c="grey", alpha=1, res=120):
+    def __init__(self, pos=(0,0,0), r=1, c="grey", alpha=1, res=120):
 
         if len(pos) == 2:
             pos = (pos[0], pos[1], 0)
         Polygon.__init__(self, pos, nsides=res, r=r)
-        self.wireframe(not fill).alpha(alpha).c(c)
+        self.alpha(alpha).c(c)
         self.name = "Circle"
 
 
-def Star(pos=(0, 0, 0), n=5, r1=0.7, r2=1.0, line=False, c="lb", alpha=1):
+class Star(Mesh):
     """
     Build a 2D star shape of `n` cusps of inner radius `r1` and outer radius `r2`.
 
@@ -1144,35 +1188,38 @@ def Star(pos=(0, 0, 0), n=5, r1=0.7, r2=1.0, line=False, c="lb", alpha=1):
 
     |extrude| |extrude.py|_
     """
-    ps = vtk.vtkRegularPolygonSource()
-    ps.SetNumberOfSides(n)
-    ps.SetRadius(r2)
-    ps.Update()
-    pts = vtk_to_numpy(ps.GetOutput().GetPoints().GetData())
+    def __init__(self, pos=(0,0,0), n=5, r1=0.7, r2=1.0, line=False, c="lb", alpha=1):
 
-    apts=[]
-    for i,p in enumerate(pts):
-        apts.append(p)
-        if i+1<n:
-            apts.append((p+pts[i+1])/2*r1/r2)
-    apts.append((pts[-1]+pts[0])/2*r1/r2)
+        if len(pos) == 2:
+            pos = (pos[0], pos[1], 0)
 
-    if line:
-        apts.append(pts[0])
-        mesh = Line(apts).c(c).alpha(alpha)
-    else:
-        apts.append((0,0,0))
-        cells=[]
-        for i in range(2*n-1):
-            cell = [2*n, i, i+1]
-            cells.append(cell)
-        cells.append([2*n, i+1, 0])
-        mesh = Mesh([apts, cells], c, alpha)
+        t = np.linspace(np.pi/2, 5/2*np.pi, num=n, endpoint=False)
+        x, y = utils.pol2cart(np.ones_like(t)*r2, t)
+        pts = np.c_[x,y, np.zeros_like(x)]
 
-    mesh.SetPosition(pos)
-    settings.collectable_actors.append(mesh)
-    mesh.name = "Star"
-    return mesh
+        apts=[]
+        for i,p in enumerate(pts):
+            apts.append(p)
+            if i+1<n:
+                apts.append((p+pts[i+1])/2*r1/r2)
+        apts.append((pts[-1]+pts[0])/2*r1/r2)
+
+        if line:
+            apts.append(pts[0])
+            #mesh = Line(apts).c(c).alpha(alpha)
+            Mesh.__init__(self, [apts, list(range(len(apts)))], c, alpha)
+        else:
+            apts.append((0,0,0))
+            cells=[]
+            for i in range(2*n-1):
+                cell = [2*n, i, i+1]
+                cells.append(cell)
+            cells.append([2*n, i+1, 0])
+            Mesh.__init__(self, [apts, cells], c, alpha)
+
+        self.SetPosition(pos)
+        settings.collectable_actors.append(self)
+        self.name = "Star"
 
 
 class Disc(Mesh):
@@ -1190,6 +1237,8 @@ class Disc(Mesh):
         res=12,
         resphi=None,
     ):
+        if len(pos) == 2:
+            pos = (pos[0], pos[1], 0)
         ps = vtk.vtkDiskSource()
         ps.SetInnerRadius(r1)
         ps.SetOuterRadius(r2)
@@ -1226,6 +1275,10 @@ class Arc(Mesh):
         alpha=1,
         res=48,
     ):
+        if len(point1) == 2:
+            point1 = (point1[0], point1[1], 0)
+        if len(point2) == 2:
+            point2 = (point2[0], point2[1], 0)
         ar = vtk.vtkArcSource()
         if point2 is not None:
             ar.UseNormalAndAngleOff()
@@ -1238,7 +1291,7 @@ class Arc(Mesh):
             ar.SetPolarVector(point1)
             ar.SetNormal(normal)
         else:
-            printc("Error in Arc(): incorrect input.")
+            printc("Error in Arc(): incorrect input.", c=1)
             return None
         ar.SetNegative(invert)
         ar.SetResolution(res)
@@ -1252,17 +1305,46 @@ class Arc(Mesh):
 class Sphere(Mesh):
     """Build a sphere at position `pos` of radius `r`.
 
-    |Sphere|
+    :param r float: sphere radius
+    :param int res: resolution in phi, resolution in theta is 2*res
+    :param bool quads: sphere mesh will be made of quads instead of triangles
+
+    |Sphere| |sphericgrid|
     """
-    def __init__(self, pos=(0, 0, 0), r=1, c="r", alpha=1, res=24):
+    def __init__(self, pos=(0, 0, 0), r=1, c="r", alpha=1, res=24, quads=False):
 
-        ss = vtk.vtkSphereSource()
-        ss.SetRadius(r)
-        ss.SetThetaResolution(2 * res)
-        ss.SetPhiResolution(res)
-        ss.Update()
+        if quads:
+            if res<4: res=4
+            img = vtk.vtkImageData()
+            img.SetDimensions(res-1,res-1,res-1)
+            rs = 1./(res-2)
+            img.SetSpacing(rs,rs,rs)
+            gf = vtk.vtkGeometryFilter()
+            gf.SetInputData(img)
+            gf.Update()
+            Mesh.__init__(self, gf.GetOutput(), c, alpha)
+            self.lw(0.1)
 
-        Mesh.__init__(self, ss.GetOutput(), c, alpha)
+            cgpts = self.points() - (0.5,0.5,0.5)
+
+            x, y, z = cgpts.T
+            x = x*(1+x*x)/2
+            y = y*(1+y*y)/2
+            z = z*(1+z*z)/2
+            _, theta, phi = utils.cart2spher(x, y, z)
+
+            pts = utils.spher2cart(np.ones_like(phi)*r, theta, phi)
+            self.points(pts)
+
+        else:
+
+            ss = vtk.vtkSphereSource()
+            ss.SetRadius(r)
+            ss.SetPhiResolution(res)
+            ss.SetThetaResolution(2 * res)
+            ss.Update()
+
+            Mesh.__init__(self, ss.GetOutput(), c, alpha)
 
         self.phong()
         self.SetPosition(pos)
@@ -1358,29 +1440,6 @@ class Spheres(Mesh):
         self.name = "Spheres"
 
 
-def SphericGrid(pos=(0, 0, 0), r=1, c="t", alpha=1, res=12):
-    """Build a sphere made of quads.
-
-    |sphericgrid|
-    """
-    sg = CubicGrid(n=(res,res,res))
-
-    cgpts = sg.points()-(.5,.5,.5)
-
-    x, y, z = cgpts[:,0], cgpts[:,1], cgpts[:,2]
-    x = x*(1+x*x)/2
-    y = y*(1+y*y)/2
-    z = z*(1+z*z)/2
-    _, theta, phi = utils.cart2spher(x, y, z)
-
-    pts = utils.spher2cart(np.ones_like(phi)*r, theta, phi)
-
-    sg.points(pts).computeNormals().c(c).alpha(alpha)
-    sg.SetPosition(pos)
-    sg.name = "SphericGrid"
-    return sg
-
-
 class Earth(Mesh):
     """Build a textured mesh representing the Earth.
 
@@ -1447,7 +1506,6 @@ class Ellipsoid(Mesh):
 
         Mesh.__init__(self, pd, c, alpha)
         self.phong()
-
         self.GetProperty().BackfaceCullingOn()
         self.SetPosition(pos)
         self.Length = -np.array(axis1) / 2 + pos
@@ -1491,6 +1549,9 @@ class Grid(Mesh):
                 resy=10,
                 ):
 
+        if len(pos) == 2:
+            pos = (pos[0], pos[1], 0)
+
         if utils.isSequence(sx) and utils.isSequence(sy):
             verts = []
             for y in sy:
@@ -1512,24 +1573,12 @@ class Grid(Mesh):
             ps.Update()
             poly0 = ps.GetOutput()
             t0 = vtk.vtkTransform()
-            #            t0.Translate(pos)
             t0.Scale(sx, sy, 1)
             tf0 = vtk.vtkTransformPolyDataFilter()
             tf0.SetInputData(poly0)
             tf0.SetTransform(t0)
             tf0.Update()
             poly = tf0.GetOutput()
-            #            axis = np.array(normal) / np.linalg.norm(normal)
-            #            theta = np.arccos(axis[2])
-            #            phi = np.arctan2(axis[1], axis[0])
-            #            t = vtk.vtkTransform()
-            #            t.PostMultiply()
-            #            t.RotateY(np.rad2deg(theta))
-            #            t.RotateZ(np.rad2deg(phi))
-            #            tf = vtk.vtkTransformPolyDataFilter()
-            #            tf.SetInputData(poly)
-            #            tf.SetTransform(t)
-            #            tf.Update()
             Mesh.__init__(self, poly, c, alpha)
             self.SetPosition(pos)
 
@@ -1549,6 +1598,9 @@ class Plane(Mesh):
     """
     def __init__(self, pos=(0, 0, 0), normal=(0, 0, 1), sx=1, sy=None, c="g", alpha=1):
 
+        if len(pos) == 2:
+            pos = (pos[0], pos[1], 0)
+
         if sy is None:
             sy = sx
         ps = vtk.vtkPlaneSource()
@@ -1576,14 +1628,24 @@ class Plane(Mesh):
         self.top = np.array(normal)
         self.bottom = np.array([0,0,0])
 
+
 def Rectangle(p1=(0, 0, 0), p2=(2, 1, 0), lw=1, c="g", alpha=1):
     """Build a rectangle in the xy plane identified by two corner points."""
-    p1 = np.array(p1)
-    p2 = np.array(p2)
-    pos = (p1 + p2) / 2
-    length = abs(p2[0] - p1[0])
-    height = abs(p2[1] - p1[1])
-    mesh = Plane(pos, [0,0,1], length, height, c, alpha)
+    if len(p1) == 2:
+        p1 = np.array([p1[0], p1[1], 0.])
+    else:
+        p1 = np.array(p1)
+    if len(p2) == 2:
+        p2 = np.array([p2[0], p2[1], 0.])
+    else:
+        p2 = np.array(p2)
+    p1r = np.array([p2[0], p1[1], 0.])
+    p2l = np.array([p1[0], p2[1], 0.])
+    pts = ([0,0,0], p1r-p1 , p2-p1, p2l-p1)
+    faces = [(0,1,2,3)]
+    mesh = Mesh([pts, faces], c, alpha)
+    mesh.SetOrigin((p1r+p1)/2)
+    mesh.SetPosition(p1)
     mesh.name = "Rectangle"
     return mesh
 
@@ -1620,26 +1682,22 @@ def Cube(pos=(0, 0, 0), side=1, c="g", alpha=1):
     return mesh
 
 def CubicGrid(pos=(0, 0, 0), n=(10,10,10), spacing=(), c="lightgrey", alpha=0.1):
-    """Build a cubic Mesh made o `n` small cubes in the 3 axis directions.
+    """Build a cubic Mesh made o `n` small quads in the 3 axis directions.
 
     :param list pos: position of the left bottom corner
     :param int n: number of subdivisions
-    :parameter list spacing: size of the side of the cube in the 3 directions
+    :parameter list spacing: size of the side of the single quad in the 3 directions
     """
-    from vtkplotter.volume import Volume
-    n = np.array(n).astype(int) + 2
-    nx, ny, nz = n
-    mx, my, mz = n - 1
-    data_matrix = np.zeros([nx, ny, nz], dtype=np.uint8)
-    data_matrix[0:mx, 0:my, 0:mz] = 1
-    data_matrix[mx:nx, my:ny, mz:nz] = 2
-    if len(spacing) == 0:
-        spacing = 1/(n-2)
-    vol = Volume(data_matrix, spacing=spacing)
-    mesh = vol.legosurface().clean().c(c).alpha(alpha)
+    img = vtk.vtkImageData()
+    img.SetDimensions(n[0]+1,n[1]+1,n[2]+1)
+    if len(spacing)==3:
+        img.SetSpacing(spacing)
+    else:
+        img.SetSpacing(1./n[0], 1./n[1], 1./n[2])
+    mesh = utils.geometry(img).c(c).alpha(alpha)
     mesh.SetPosition(pos)
-    mesh.base = np.array([0,0,0])
-    mesh.top = np.array([0,0,1])
+    mesh.base = np.array([0.5,0.5,0])
+    mesh.top  = np.array([0.5,0.5,1])
     mesh.name = "CubicGrid"
     return mesh
 
@@ -1840,7 +1898,7 @@ class Paraboloid(Mesh):
         contours.Update()
 
         Mesh.__init__(self, contours.GetOutput(), c, alpha)
-        self.flipNormals().phong()
+        self.computeNormals().phong()
         self.mapper().ScalarVisibilityOff()
         self.SetPosition(pos)
         settings.collectable_actors.append(self)
@@ -1872,52 +1930,135 @@ class Hyperboloid(Mesh):
         contours.Update()
 
         Mesh.__init__(self, contours.GetOutput(), c, alpha)
-        self.flipNormals().phong()
+        self.computeNormals().phong()
         self.mapper().ScalarVisibilityOff()
         self.SetPosition(pos)
         settings.collectable_actors.append(self)
         self.name = "Hyperboloid"
 
 
-def Text(
-    txt,
-    pos="top-left",
-    s=1,
-    depth=0.1,
-    justify="bottom-left",
-    c=None,
-    alpha=1,
-    bc=None,
-    bg=None,
-    font="courier",
-):
+class Text(Mesh):
     """
-    Returns a polygonal ``Mesh`` that shows a 2D/3D text.
+    Returns a polygonal ``Mesh`` of 3D text.
 
-    :param pos: position in 3D space,
-                a 2D text is placed in one of the 8 positions:
-
-                    1, bottom-left
-                    2, bottom-right
-                    3, top-left
-                    4, top-right
-                    5, bottom-middle
-                    6, middle-right
-                    7, middle-left
-                    8, top-middle
-
-                If a pair (x,y) is passed as input the 2D text is place at that
-                position in the coordinate system of the 2D screen (with the
-                origin sitting at the bottom left).
-
-    :type pos: list, int
+    :param list pos: position coordinates in 3D space
     :param float s: size of text.
     :param float depth: text thickness.
     :param str justify: text justification
         (bottom-left, bottom-right, top-left, top-right, centered).
-    :param bg: background color of corner annotations. Only applies of `pos` is ``int``.
-    :param str font: additional available fonts are:
 
+    :param bc: back face color of text (not background)
+
+    |markpoint| |markpoint.py|_
+    """
+
+    def __init__(self,
+                txt,
+                pos=(0,0,0),
+                s=1,
+                depth=0,
+                justify="bottom-left",
+                c=None,
+                alpha=1,
+                bc=None,
+                ):
+
+        if isinstance(pos, str):
+            printc("Please use Text2D() instead of Text() for 2D text and corner annotations.", c=1)
+            raise RuntimeError()
+
+        if len(pos)==2:
+            pos = (pos[0], pos[1], 0)
+
+        if c is None: # automatic black or white
+            pli = settings.plotter_instance
+            if pli and pli.renderer:
+                c = (0.9, 0.9, 0.9)
+                if np.sum(pli.renderer.GetBackground()) > 1.5:
+                    c = (0.1, 0.1, 0.1)
+            else:
+                c = (0.6, 0.6, 0.6)
+
+        # otherwise build the 3D text, fonts do not apply
+        tt = vtk.vtkVectorText()
+        tt.SetText(str(txt))
+        tt.Update()
+        tpoly = tt.GetOutput()
+
+        bb = tpoly.GetBounds()
+        dx, dy = (bb[1] - bb[0]) / 2 * s, (bb[3] - bb[2]) / 2 * s
+        cm = np.array([(bb[1] + bb[0]) / 2,
+                       (bb[3] + bb[2]) / 2,
+                       (bb[5] + bb[4]) / 2]) * s
+        shift = -cm
+        if "bottom" in justify: shift += np.array([  0, dy, 0])
+        if "top"    in justify: shift += np.array([  0,-dy, 0])
+        if "left"   in justify: shift += np.array([ dx,  0, 0])
+        if "right"  in justify: shift += np.array([-dx,  0, 0])
+
+        t = vtk.vtkTransform()
+        t.Translate(shift)
+        t.Scale(s, s, s)
+        tf = vtk.vtkTransformPolyDataFilter()
+        tf.SetInputData(tpoly)
+        tf.SetTransform(t)
+        tf.Update()
+        tpoly = tf.GetOutput()
+
+        if depth:
+            extrude = vtk.vtkLinearExtrusionFilter()
+            extrude.SetInputData(tpoly)
+            extrude.SetExtrusionTypeToVectorExtrusion()
+            extrude.SetVector(0, 0, 1)
+            extrude.SetScaleFactor(depth*dy)
+            extrude.Update()
+            tpoly = extrude.GetOutput()
+        Mesh.__init__(self, tpoly, c, alpha)
+        if bc is not None:
+            self.backColor(bc)
+        self.SetPosition(pos)
+        settings.collectable_actors.append(self)
+        self.name = "Text"
+
+
+def Text2D(
+    txt,
+    pos=3,
+    s=1,
+    c=None,
+    alpha=1,
+    bg=None,
+    font="courier",
+    justify="bottom-left",
+):
+    """Returns a ``vtkActor2D`` representing 2D text.
+
+    :param pos: text is placed in one of the 8 positions:
+
+            1, bottom-left
+            2, bottom-right
+            3, top-left
+            4, top-right
+            5, bottom-middle
+            6, middle-right
+            7, middle-left
+            8, top-middle
+
+        If a pair (x,y) is passed as input the 2D text is place at that
+        position in the coordinate system of the 2D screen (with the
+        origin sitting at the bottom left).
+
+    :type pos: list, int
+
+    :param float s: size of text.
+    :param bg: background color
+    :param float alpha: background opacity
+    :param str justify: text justification
+    :param str font: available fonts are
+
+            - Courier
+            - Times
+            - Arial
             - Ageo
             - Aldora
             - CallingCode
@@ -1935,16 +2076,15 @@ def Text(
             - SchoolTeacher
             - SpecialElite
 
-        Font choice does not apply for 3D text.
         A path to `otf` or `ttf` font-file can also be supplied as input.
 
         All fonts are free for personal use.
         Check out conditions in `vtkplotter/fonts/licenses` for commercial use
         and: https://www.1001freefonts.com
 
-    .. hint:: Examples, |fonts.py|_ |colorcubes.py|_ |markpoint.py|_ |annotations.py|_
+    .. hint:: Examples, |fonts.py|_ |colorcubes.py|_ |annotations.py|_
 
-        |colorcubes| |markpoint|
+        |colorcubes|
 
         |fonts|
     """
@@ -1954,7 +2094,7 @@ def Text(
             if np.sum(settings.plotter_instance.renderer.GetBackground()) > 1.5:
                 c = (0.1, 0.1, 0.1)
         else:
-            c = (0.5, 0.5, 0.5)
+            c = (0.3, 0.3, 0.3)
 
     if isinstance(pos, str): # corners
         if "top" in pos:
@@ -1986,7 +2126,7 @@ def Text(
         elif font.lower() == "arial": cap.SetFontFamilyToArial()
         else:
             cap.SetFontFamily(vtk.VTK_FONT_FILE)
-            cap.SetFontFile(settings.fonts_path+font+'.ttf')
+            cap.SetFontFile(settings.fonts_path + font +'.ttf')
         if bg:
             bgcol = getColor(bg)
             cap.SetBackgroundColor(bgcol)
@@ -1997,85 +2137,56 @@ def Text(
         settings.collectable_actors.append(ca)
         return ca
 
-    elif len(pos)==2: # passing (x,y) coords
-        actor2d = vtk.vtkActor2D()
-        actor2d.SetPosition(pos)
-        tmapper = vtk.vtkTextMapper()
-        actor2d.SetMapper(tmapper)
-        tp = tmapper.GetTextProperty()
-        tp.BoldOff()
-        tp.SetFontSize(s*20)
-        tp.SetColor(getColor(c))
-        tp.SetJustificationToLeft()
+    if len(pos)!=2:
+        print("Error in Text2D(): len(pos) must be 2 or integer value.")
+        raise RuntimeError()
+
+    actor2d = vtk.vtkActor2D()
+    actor2d.SetPosition(pos)
+    tmapper = vtk.vtkTextMapper()
+    tmapper.SetInput(str(txt))
+    actor2d.SetMapper(tmapper)
+    tp = tmapper.GetTextProperty()
+    tp.BoldOff()
+    tp.SetFontSize(s*20)
+    tp.SetColor(getColor(c))
+    tp.SetJustificationToLeft()
+    if "top" in justify:
+        tp.SetVerticalJustificationToTop()
+    if "bottom" in justify:
         tp.SetVerticalJustificationToBottom()
-        if font.lower() == "courier": tp.SetFontFamilyToCourier()
-        elif font.lower() == "times": tp.SetFontFamilyToTimes()
-        elif font.lower() == "arial": tp.SetFontFamilyToArial()
-        else:
-            tp.SetFontFamily(vtk.VTK_FONT_FILE)
-            import os
-            if font in settings.fonts:
-                tp.SetFontFile(settings.fonts_path + font + '.ttf')
-            elif os.path.exists(font):
-                tp.SetFontFile(font)
-            else:
-                printc("~sad Font", font, "not found in", settings.fonts_path, c="r")
-                printc("~pin Available fonts are:", settings.fonts, c="m")
-                return None
-        if bg:
-            bgcol = getColor(bg)
-            tp.SetBackgroundColor(bgcol)
-            tp.SetBackgroundOpacity(alpha * 0.5)
-            tp.SetFrameColor(bgcol)
-            tp.FrameOn()
-        tmapper.SetInput(str(txt))
-        actor2d.PickableOff()
-        setattr(actor2d, 'renderedAt', set())
-        settings.collectable_actors.append(actor2d)
-        return actor2d
+    if "cent" in justify:
+        tp.SetVerticalJustificationToCentered()
+        tp.SetJustificationToCentered()
+    if "left" in justify:
+        tp.SetJustificationToLeft()
+    if "right" in justify:
+        tp.SetJustificationToRight()
 
+    if font.lower() == "courier": tp.SetFontFamilyToCourier()
+    elif font.lower() == "times": tp.SetFontFamilyToTimes()
+    elif font.lower() == "arial": tp.SetFontFamilyToArial()
     else:
-        # otherwise build the 3D text, fonts do not apply
-        tt = vtk.vtkVectorText()
-        tt.SetText(str(txt))
-        tt.Update()
-        tpoly = tt.GetOutput()
-
-        bb = tpoly.GetBounds()
-        dx, dy = (bb[1] - bb[0]) / 2 * s, (bb[3] - bb[2]) / 2 * s
-        cm = np.array([(bb[1] + bb[0]) / 2, (bb[3] + bb[2]) / 2, (bb[5] + bb[4]) / 2]) * s
-        shift = -cm
-        if "bottom" in justify: shift += np.array([  0, dy, 0])
-        if "top"    in justify: shift += np.array([  0,-dy, 0])
-        if "left"   in justify: shift += np.array([ dx,  0, 0])
-        if "right"  in justify: shift += np.array([-dx,  0, 0])
-
-        t = vtk.vtkTransform()
-        t.Translate(shift)
-        t.Scale(s, s, s)
-        tf = vtk.vtkTransformPolyDataFilter()
-        tf.SetInputData(tpoly)
-        tf.SetTransform(t)
-        tf.Update()
-        tpoly = tf.GetOutput()
-
-        if depth:
-            extrude = vtk.vtkLinearExtrusionFilter()
-            extrude.SetInputData(tpoly)
-            extrude.SetExtrusionTypeToVectorExtrusion()
-            extrude.SetVector(0, 0, 1)
-            extrude.SetScaleFactor(depth*dy)
-            extrude.Update()
-            tpoly = extrude.GetOutput()
-        ttmesh = Mesh(tpoly, c, alpha)
-        if bc is not None:
-            ttmesh.backColor(bc)
-
-        ttmesh.SetPosition(pos)
-        settings.collectable_actors.append(ttmesh)
-        ttmesh.name = "Text"
-        return ttmesh
-
+        tp.SetFontFamily(vtk.VTK_FONT_FILE)
+        import os
+        if font in settings.fonts:
+            tp.SetFontFile(settings.fonts_path + font + '.ttf')
+        elif os.path.exists(font):
+            tp.SetFontFile(font)
+        else:
+            printc("~sad Font", font, "not found in", settings.fonts_path, c="r")
+            printc("~pin Available fonts are:", settings.fonts, c="m")
+            return None
+    if bg:
+        bgcol = getColor(bg)
+        tp.SetBackgroundColor(bgcol)
+        tp.SetBackgroundOpacity(alpha * 0.5)
+        tp.SetFrameColor(bgcol)
+        tp.FrameOn()
+    actor2d.PickableOff()
+    setattr(actor2d, 'renderedAt', set())
+    settings.collectable_actors.append(actor2d)
+    return actor2d
 
 class Latex(Picture):
     """
@@ -2104,6 +2215,8 @@ class Latex(Picture):
         usetex=False,
         fromweb=False,
     ):
+        if len(pos) == 2:
+            pos = (pos[0], pos[1], 0)
         try:
 
             def build_img_web(formula, tfile):
